/*
this module is supposed to take the images from the SDRAM and then process them into
blocks. blocks are 16x16 areas of pixels that represent the window for our correlator. 

We will load 1 16x16 block from the image at t=1 
and we will load every combination in a search space of 24x24 for t=2


Aaron Nathan
Jimmy Chen
ECE576 Final Project
*/


module BlockSAD
(
	iReset,
	iClock,
	oProcessing,			//high as long as we are working with the data, this should freeze our ram input
	oLED,
	oSADTemp,
	
	//the images from the SDRAM
	oSDRAM_READ_CLOCK,  //data clock on the sdram (each cycle gets a new word)
	oSDRAM_READ_LOGIC,  //set high when data is wanted
	iSDRAM_IMAGE1,		   //data from image 1
	iSDRAM_IMAGE1_EMPTY,
	iSDRAM_IMAGE2,		   //data from image 2
	iCCDFRAMECOUNT,
	iCCDCLOCK,
	//the output to the VGA
	oVGASRAM_ADDR,	//address of VGA output SRAM (320x240)
	oVGASRAM_DATA,	//data for		 "			"
	iVGA_OK_TO_WRITE,		//signal indicating it is OK to write to the SRAM	
	iVGA_CTRL_CLK
);
	input   		iReset;
	input 			iClock;
	output 			oProcessing;			//high as long as we are working with the data, this should freeze our ram input
	output [17:0]	oLED;
	output [15:0]	oSADTemp;
	//the images from the SDRAM
	output 			oSDRAM_READ_CLOCK;  	//data clock on the sdram (each cycle gets a new word)
	output 			oSDRAM_READ_LOGIC; 		//set high when data is wanted
	input [15:0]	iSDRAM_IMAGE1;		   	//data from image 1
	input 			iSDRAM_IMAGE1_EMPTY;	// indicates the FIFO buffer is empty
	input [15:0]	iSDRAM_IMAGE2;		   	//data from image 2
	input [31:0]   	iCCDFRAMECOUNT;         // the framecount
	input			iCCDCLOCK;
	//the output to the VGA
	output [17:0]	oVGASRAM_ADDR;		//address of VGA output SRAM (320x240)
	output [15:0]	oVGASRAM_DATA;		//data for		 "			"
	input 			iVGA_OK_TO_WRITE;		//signal indicating it is OK to write to the SRAM	
	input			iVGA_CTRL_CLK;

//---------------------------------------------------------------------------------------------------------------------------

parameter	SADBITS	 	= 1;
parameter	SADAPPEND	= 14'b0;

	reg [9:0] Im1XcoordStart;	
	reg [9:0] Im1YcoordStart;	

	reg [9:0] Im2XcoordStart;	
	reg [9:0] Im2YcoordStart;	
	
	reg [9:0] BestMatchXcoordStart;	
	reg [9:0] BestMatchYcoordStart;
	
	reg [9:0] ImXcoord;	//the coordinates beeing read out of the sdram based images
	reg [9:0] ImYcoord;
		
	reg [17:0] rLED;

	reg [17:0] rVGASRAM_ADDR;
	reg [15:0] rVGASRAM_DATA;
	
	reg [SADBITS:0] 	RefPixels  	[7:0][7:0];				//The block we are trying to match. 10 bit pixel, 8 by 8 pixel
	reg [SADBITS:0] 	CompPixels 	[23:0][23:0];			//The search region, each is 10 bit pixel, 23 by 23 pixel
	reg [7:0] 	SumsAbsDiffs [15:0][15:0];			//The results of the SAD calculation.
	
	reg [3:0]	state;	//state machine
	
	reg 		FrameGrabDone;
	reg			ProcessingDone;
	
	//block offsets for SAD
	reg [4:0]  SX;
	reg [4:0]  SY;

	assign oSADTemp = SADTemp;
	reg [15:0] SADTemp;
	reg [15:0] SADMin; 
	reg [4:0]  SXMin;
	reg [4:0]  SYMin;


	//the max size is 1024 * 64 =65K ~ add some for safety (not really needed at all)
													
	assign oSDRAM_READ_LOGIC = iVGA_OK_TO_WRITE;
	assign oSDRAM_READ_CLOCK = iVGA_CTRL_CLK;
	assign oVGASRAM_ADDR = rVGASRAM_ADDR;
	assign oVGASRAM_DATA = rVGASRAM_DATA;		
	assign  oLED = rLED;
	assign 	oProcessing = 0;
	
	reg doneproc;
	
//state names
parameter grab 	= 4'd0;
parameter calcsumabs  	= 4'd1;
parameter calcsumabs2  	= 4'd2;
parameter done			= 4'd3;

	always @ (posedge oSDRAM_READ_CLOCK)
	begin
		//the ram is a FIFO buffer, so we will need to scan and selectively add the correct pixels.
		if (iReset)			
		begin
			ImXcoord<=10'd0;
			ImYcoord<=10'd0;
			//these are the real deal
			//REFERNECE
			Im1XcoordStart<=10'd321; //must be odd to show up on screen 
			Im1YcoordStart<=10'd241;
			Im2XcoordStart<=10'd313;
			Im2YcoordStart<=10'd233;
			BestMatchXcoordStart<=10'd313;
			BestMatchYcoordStart<=10'd233;
			
			state<=grab;
			SADMin<=16'hFFFF;
			SXMin<=7;
			SYMin<=7;
			SX<=0;
			SY<=0;
			doneproc<=0;
		end
		
		else if (iVGA_OK_TO_WRITE)  //modify display during sync
		begin
			
			
			rVGASRAM_ADDR <= { ImXcoord[9:1],  ImYcoord[9:1]};
			

				
			//draw a red box
			//top and bottom
			if ((ImXcoord >= Im1XcoordStart)  	&& 
				(ImXcoord <= Im1XcoordStart+8)	&&
				((ImYcoord == Im1YcoordStart) || (ImYcoord == Im1YcoordStart+8)))				
				rVGASRAM_DATA <=  {iSDRAM_IMAGE1[9:0], 6'b100000}; 
			
			//left and right
			else if ((ImYcoord >= Im1YcoordStart)  	&& 
				(ImYcoord <= Im1YcoordStart+8)		&&
				((ImXcoord == Im1XcoordStart) || (ImXcoord == Im1XcoordStart+8)))				
				rVGASRAM_DATA <=  {iSDRAM_IMAGE1[9:0], 6'b100000}; 
					
			//these are the minimal offset parameters from the upper left corner of the reference		
			//reg [4:0]  SXMin;
			//reg [4:0]  SYMin;						
			//draw a green box around the minimum area
			//top and bottom
			else if ((ImXcoord >= BestMatchXcoordStart+SXMin)  	&& 
				(ImXcoord <= BestMatchXcoordStart+SXMin+8)		&&
				(ImYcoord >= BestMatchYcoordStart+SYMin) 		&&
			    (ImYcoord <= BestMatchYcoordStart+SYMin+8))				
				rVGASRAM_DATA <=  {iSDRAM_IMAGE1[9:0], 6'b010000}; 
				
				
			//draw a blue box around the candidate area
			//top and bottom
			else if ((ImXcoord >= BestMatchXcoordStart-2)  	&& 
				(ImXcoord <= BestMatchXcoordStart+26)		&&
				((ImYcoord == BestMatchYcoordStart-2) || (ImYcoord == BestMatchYcoordStart+26)))				
				rVGASRAM_DATA <=  {iSDRAM_IMAGE1[9:0], 6'b001000}; 
			
			//left and right
			else if ((ImYcoord >= BestMatchYcoordStart-2)  	&& 
				(ImYcoord <= BestMatchYcoordStart+26)		&&
				((ImXcoord == BestMatchXcoordStart-2) || (ImXcoord == BestMatchXcoordStart+26)))				
				rVGASRAM_DATA <=  {iSDRAM_IMAGE1[9:0], 6'b001000}; 
				
																													
			else if (ImYcoord < 400)
				rVGASRAM_DATA <=  {iSDRAM_IMAGE2[9:0], 6'b000000};  
			
			else if (	(ImYcoord >= 410) && 
						(ImYcoord <= 433) &&
						(ImXcoord >= 10 ) &&
						(ImXcoord <= 33))
				rVGASRAM_DATA <=  {CompPixels [ImXcoord-10][ImYcoord-410],SADAPPEND};
				
			else if (	(ImYcoord >= 410) && 
						(ImYcoord <= 417) &&
						(ImXcoord >= 50 ) &&
						(ImXcoord <= 57))
				rVGASRAM_DATA <=  {RefPixels[ImXcoord-50][ImYcoord-410],SADAPPEND};
			else
				rVGASRAM_DATA <=  16'h0;
			//------------------------------------------------------------------------
			//------------------------------------------------------------------------
			//------------------------------------------------------------------------
			//------------------------------------------------------------------------
			//------------------------------------------------------------------------			
			//------------------------------------------------------------------------
					
			//Begin SAD state machine
			
			case(state)
				grab:
				begin
					
				rLED[17:16]<=2'b00;
				//load our reference picture
				if ((ImXcoord >= Im1XcoordStart) &&
					(ImXcoord <= Im1XcoordStart+7) &&
					(ImYcoord >= Im1YcoordStart) &&
					(ImYcoord <= Im1YcoordStart+7))
				begin
					RefPixels[ImXcoord-Im1XcoordStart][ImYcoord-Im1YcoordStart] <= iSDRAM_IMAGE1[9:(9-SADBITS)]; 
				end
				
				//load our candidate match area
				if ((ImXcoord >= Im2XcoordStart) &&
					(ImXcoord <= Im2XcoordStart+23) &&
					(ImYcoord >= Im2YcoordStart) &&
					(ImYcoord <= Im2YcoordStart+23))
				begin
					CompPixels[ImXcoord-Im2XcoordStart][ImYcoord-Im2YcoordStart] <= iSDRAM_IMAGE2[9:(9-SADBITS)]; 
				end
										
				if (ImYcoord == 479)  //done loading the registers up
				begin
					state<=calcsumabs;							
					SADMin<=16'hFFFF;//reset
					SX<=0; SY<=0; //reset the summers					
				end
				else
					state<=grab;
				end			
					
				calcsumabs:			
				begin
				rLED[17:16]<=2'b01;				
					if (SX<15)
						SX<=SX+1;
					else
					begin
						SX<=0;
						if (SY<15)
							SY<=SY+1;
						else
						begin
							SY<=0;							
							doneproc<=1;
						end
					end
					
					rLED[0]<=~rLED[0];
					
					SADTemp <= (
					 (RefPixels[0][0]>CompPixels [(0+SX)][(0+SY)])?(RefPixels[0][0]-CompPixels [(0+SX)][(0+SY)]):(CompPixels [(0+SX)][(0+SY)]-RefPixels[0][0])
					+(RefPixels[0][1]>CompPixels [(0+SX)][(1+SY)])?(RefPixels[0][1]-CompPixels [(0+SX)][(1+SY)]):(CompPixels [(0+SX)][(1+SY)]-RefPixels[0][1])
					+(RefPixels[0][2]>CompPixels [(0+SX)][(2+SY)])?(RefPixels[0][2]-CompPixels [(0+SX)][(2+SY)]):(CompPixels [(0+SX)][(2+SY)]-RefPixels[0][2])
					+(RefPixels[0][3]>CompPixels [(0+SX)][(3+SY)])?(RefPixels[0][3]-CompPixels [(0+SX)][(3+SY)]):(CompPixels [(0+SX)][(3+SY)]-RefPixels[0][3])
					+(RefPixels[0][4]>CompPixels [(0+SX)][(4+SY)])?(RefPixels[0][4]-CompPixels [(0+SX)][(4+SY)]):(CompPixels [(0+SX)][(4+SY)]-RefPixels[0][4])
					+(RefPixels[0][5]>CompPixels [(0+SX)][(5+SY)])?(RefPixels[0][5]-CompPixels [(0+SX)][(5+SY)]):(CompPixels [(0+SX)][(5+SY)]-RefPixels[0][5])
					+(RefPixels[0][6]>CompPixels [(0+SX)][(6+SY)])?(RefPixels[0][6]-CompPixels [(0+SX)][(6+SY)]):(CompPixels [(0+SX)][(6+SY)]-RefPixels[0][6])
					+(RefPixels[0][7]>CompPixels [(0+SX)][(7+SY)])?(RefPixels[0][7]-CompPixels [(0+SX)][(7+SY)]):(CompPixels [(0+SX)][(7+SY)]-RefPixels[0][7])
					
					+(RefPixels[1][0]>CompPixels [(1+SX)][(0+SY)])?(RefPixels[1][0]-CompPixels [(1+SX)][(0+SY)]):(CompPixels [(1+SX)][(0+SY)]-RefPixels[1][0])
					+(RefPixels[1][1]>CompPixels [(1+SX)][(1+SY)])?(RefPixels[1][1]-CompPixels [(1+SX)][(1+SY)]):(CompPixels [(1+SX)][(1+SY)]-RefPixels[1][1])
					+(RefPixels[1][2]>CompPixels [(1+SX)][(2+SY)])?(RefPixels[1][2]-CompPixels [(1+SX)][(2+SY)]):(CompPixels [(1+SX)][(2+SY)]-RefPixels[1][2])
					+(RefPixels[1][3]>CompPixels [(1+SX)][(3+SY)])?(RefPixels[1][3]-CompPixels [(1+SX)][(3+SY)]):(CompPixels [(1+SX)][(3+SY)]-RefPixels[1][3])
					+(RefPixels[1][4]>CompPixels [(1+SX)][(4+SY)])?(RefPixels[1][4]-CompPixels [(1+SX)][(4+SY)]):(CompPixels [(1+SX)][(4+SY)]-RefPixels[1][4])
					+(RefPixels[1][5]>CompPixels [(1+SX)][(5+SY)])?(RefPixels[1][5]-CompPixels [(1+SX)][(5+SY)]):(CompPixels [(1+SX)][(5+SY)]-RefPixels[1][5])
					+(RefPixels[1][6]>CompPixels [(1+SX)][(6+SY)])?(RefPixels[1][6]-CompPixels [(1+SX)][(6+SY)]):(CompPixels [(1+SX)][(6+SY)]-RefPixels[1][6])
					+(RefPixels[1][7]>CompPixels [(1+SX)][(7+SY)])?(RefPixels[1][7]-CompPixels [(1+SX)][(7+SY)]):(CompPixels [(1+SX)][(7+SY)]-RefPixels[1][7])
					
					+(RefPixels[2][0]>CompPixels [(2+SX)][(0+SY)])?(RefPixels[2][0]-CompPixels [(2+SX)][(0+SY)]):(CompPixels [(2+SX)][(0+SY)]-RefPixels[2][0])
					+(RefPixels[2][1]>CompPixels [(2+SX)][(1+SY)])?(RefPixels[2][1]-CompPixels [(2+SX)][(1+SY)]):(CompPixels [(2+SX)][(1+SY)]-RefPixels[2][1])
					+(RefPixels[2][2]>CompPixels [(2+SX)][(2+SY)])?(RefPixels[2][2]-CompPixels [(2+SX)][(2+SY)]):(CompPixels [(2+SX)][(2+SY)]-RefPixels[2][2])
					+(RefPixels[2][3]>CompPixels [(2+SX)][(3+SY)])?(RefPixels[2][3]-CompPixels [(2+SX)][(3+SY)]):(CompPixels [(2+SX)][(3+SY)]-RefPixels[2][3])
					+(RefPixels[2][4]>CompPixels [(2+SX)][(4+SY)])?(RefPixels[2][4]-CompPixels [(2+SX)][(4+SY)]):(CompPixels [(2+SX)][(4+SY)]-RefPixels[2][4])
					+(RefPixels[2][5]>CompPixels [(2+SX)][(5+SY)])?(RefPixels[2][5]-CompPixels [(2+SX)][(5+SY)]):(CompPixels [(2+SX)][(5+SY)]-RefPixels[2][5])
					+(RefPixels[2][6]>CompPixels [(2+SX)][(6+SY)])?(RefPixels[2][6]-CompPixels [(2+SX)][(6+SY)]):(CompPixels [(2+SX)][(6+SY)]-RefPixels[2][6])
					+(RefPixels[2][7]>CompPixels [(2+SX)][(7+SY)])?(RefPixels[2][7]-CompPixels [(2+SX)][(7+SY)]):(CompPixels [(2+SX)][(7+SY)]-RefPixels[2][7])
					
					+(RefPixels[3][0]>CompPixels [(3+SX)][(0+SY)])?(RefPixels[3][0]-CompPixels [(3+SX)][(0+SY)]):(CompPixels [(3+SX)][(0+SY)]-RefPixels[3][0])
					+(RefPixels[3][1]>CompPixels [(3+SX)][(1+SY)])?(RefPixels[3][1]-CompPixels [(3+SX)][(1+SY)]):(CompPixels [(3+SX)][(1+SY)]-RefPixels[3][1])
					+(RefPixels[3][2]>CompPixels [(3+SX)][(2+SY)])?(RefPixels[3][2]-CompPixels [(3+SX)][(2+SY)]):(CompPixels [(3+SX)][(2+SY)]-RefPixels[3][2])
					+(RefPixels[3][3]>CompPixels [(3+SX)][(3+SY)])?(RefPixels[3][3]-CompPixels [(3+SX)][(3+SY)]):(CompPixels [(3+SX)][(3+SY)]-RefPixels[3][3])
					+(RefPixels[3][4]>CompPixels [(3+SX)][(4+SY)])?(RefPixels[3][4]-CompPixels [(3+SX)][(4+SY)]):(CompPixels [(3+SX)][(4+SY)]-RefPixels[3][4])
					+(RefPixels[3][5]>CompPixels [(3+SX)][(5+SY)])?(RefPixels[3][5]-CompPixels [(3+SX)][(5+SY)]):(CompPixels [(3+SX)][(5+SY)]-RefPixels[3][5])
					+(RefPixels[3][6]>CompPixels [(3+SX)][(6+SY)])?(RefPixels[3][6]-CompPixels [(3+SX)][(6+SY)]):(CompPixels [(3+SX)][(6+SY)]-RefPixels[3][6])
					+(RefPixels[3][7]>CompPixels [(3+SX)][(7+SY)])?(RefPixels[3][7]-CompPixels [(3+SX)][(7+SY)]):(CompPixels [(3+SX)][(7+SY)]-RefPixels[3][7])
					
					+(RefPixels[4][0]>CompPixels [(4+SX)][(0+SY)])?(RefPixels[4][0]-CompPixels [(4+SX)][(0+SY)]):(CompPixels [(4+SX)][(0+SY)]-RefPixels[4][0])
					+(RefPixels[4][1]>CompPixels [(4+SX)][(1+SY)])?(RefPixels[4][1]-CompPixels [(4+SX)][(1+SY)]):(CompPixels [(4+SX)][(1+SY)]-RefPixels[4][1])
					+(RefPixels[4][2]>CompPixels [(4+SX)][(2+SY)])?(RefPixels[4][2]-CompPixels [(4+SX)][(2+SY)]):(CompPixels [(4+SX)][(2+SY)]-RefPixels[4][2])
					+(RefPixels[4][3]>CompPixels [(4+SX)][(3+SY)])?(RefPixels[4][3]-CompPixels [(4+SX)][(3+SY)]):(CompPixels [(4+SX)][(3+SY)]-RefPixels[4][3])
					+(RefPixels[4][4]>CompPixels [(4+SX)][(4+SY)])?(RefPixels[4][4]-CompPixels [(4+SX)][(4+SY)]):(CompPixels [(4+SX)][(4+SY)]-RefPixels[4][4])
					+(RefPixels[4][5]>CompPixels [(4+SX)][(5+SY)])?(RefPixels[4][5]-CompPixels [(4+SX)][(5+SY)]):(CompPixels [(4+SX)][(5+SY)]-RefPixels[4][5])
					+(RefPixels[4][6]>CompPixels [(4+SX)][(6+SY)])?(RefPixels[4][6]-CompPixels [(4+SX)][(6+SY)]):(CompPixels [(4+SX)][(6+SY)]-RefPixels[4][6])
					+(RefPixels[4][7]>CompPixels [(4+SX)][(7+SY)])?(RefPixels[4][7]-CompPixels [(4+SX)][(7+SY)]):(CompPixels [(4+SX)][(7+SY)]-RefPixels[4][7])
					
					+(RefPixels[5][0]>CompPixels [(5+SX)][(0+SY)])?(RefPixels[5][0]-CompPixels [(5+SX)][(0+SY)]):(CompPixels [(5+SX)][(0+SY)]-RefPixels[5][0])
					+(RefPixels[5][1]>CompPixels [(5+SX)][(1+SY)])?(RefPixels[5][1]-CompPixels [(5+SX)][(1+SY)]):(CompPixels [(5+SX)][(1+SY)]-RefPixels[5][1])
					+(RefPixels[5][2]>CompPixels [(5+SX)][(2+SY)])?(RefPixels[5][2]-CompPixels [(5+SX)][(2+SY)]):(CompPixels [(5+SX)][(2+SY)]-RefPixels[5][2])
					+(RefPixels[5][3]>CompPixels [(5+SX)][(3+SY)])?(RefPixels[5][3]-CompPixels [(5+SX)][(3+SY)]):(CompPixels [(5+SX)][(3+SY)]-RefPixels[5][3])
					+(RefPixels[5][4]>CompPixels [(5+SX)][(4+SY)])?(RefPixels[5][4]-CompPixels [(5+SX)][(4+SY)]):(CompPixels [(5+SX)][(4+SY)]-RefPixels[5][4])
					+(RefPixels[5][5]>CompPixels [(5+SX)][(5+SY)])?(RefPixels[5][5]-CompPixels [(5+SX)][(5+SY)]):(CompPixels [(5+SX)][(5+SY)]-RefPixels[5][5])
					+(RefPixels[5][6]>CompPixels [(5+SX)][(6+SY)])?(RefPixels[5][6]-CompPixels [(5+SX)][(6+SY)]):(CompPixels [(5+SX)][(6+SY)]-RefPixels[5][6])
					+(RefPixels[5][7]>CompPixels [(5+SX)][(7+SY)])?(RefPixels[5][7]-CompPixels [(5+SX)][(7+SY)]):(CompPixels [(5+SX)][(7+SY)]-RefPixels[5][7])
					
					+(RefPixels[6][0]>CompPixels [(6+SX)][(0+SY)])?(RefPixels[6][0]-CompPixels [(6+SX)][(0+SY)]):(CompPixels [(6+SX)][(0+SY)]-RefPixels[6][0])
					+(RefPixels[6][1]>CompPixels [(6+SX)][(1+SY)])?(RefPixels[6][1]-CompPixels [(6+SX)][(1+SY)]):(CompPixels [(6+SX)][(1+SY)]-RefPixels[6][1])
					+(RefPixels[6][2]>CompPixels [(6+SX)][(2+SY)])?(RefPixels[6][2]-CompPixels [(6+SX)][(2+SY)]):(CompPixels [(6+SX)][(2+SY)]-RefPixels[6][2])
					+(RefPixels[6][3]>CompPixels [(6+SX)][(3+SY)])?(RefPixels[6][3]-CompPixels [(6+SX)][(3+SY)]):(CompPixels [(6+SX)][(3+SY)]-RefPixels[6][3])
					+(RefPixels[6][4]>CompPixels [(6+SX)][(4+SY)])?(RefPixels[6][4]-CompPixels [(6+SX)][(4+SY)]):(CompPixels [(6+SX)][(4+SY)]-RefPixels[6][4])
					+(RefPixels[6][5]>CompPixels [(6+SX)][(5+SY)])?(RefPixels[6][5]-CompPixels [(6+SX)][(5+SY)]):(CompPixels [(6+SX)][(5+SY)]-RefPixels[6][5])
					+(RefPixels[6][6]>CompPixels [(6+SX)][(6+SY)])?(RefPixels[6][6]-CompPixels [(6+SX)][(6+SY)]):(CompPixels [(6+SX)][(6+SY)]-RefPixels[6][6])
					+(RefPixels[6][7]>CompPixels [(6+SX)][(7+SY)])?(RefPixels[6][7]-CompPixels [(6+SX)][(7+SY)]):(CompPixels [(6+SX)][(7+SY)]-RefPixels[6][7])
					
					+(RefPixels[7][0]>CompPixels [(7+SX)][(0+SY)])?(RefPixels[7][0]-CompPixels [(7+SX)][(0+SY)]):(CompPixels [(7+SX)][(0+SY)]-RefPixels[7][0])
					+(RefPixels[7][1]>CompPixels [(7+SX)][(1+SY)])?(RefPixels[7][1]-CompPixels [(7+SX)][(1+SY)]):(CompPixels [(7+SX)][(1+SY)]-RefPixels[7][1])
					+(RefPixels[7][2]>CompPixels [(7+SX)][(2+SY)])?(RefPixels[7][2]-CompPixels [(7+SX)][(2+SY)]):(CompPixels [(7+SX)][(2+SY)]-RefPixels[7][2])
					+(RefPixels[7][3]>CompPixels [(7+SX)][(3+SY)])?(RefPixels[7][3]-CompPixels [(7+SX)][(3+SY)]):(CompPixels [(7+SX)][(3+SY)]-RefPixels[7][3])
					+(RefPixels[7][4]>CompPixels [(7+SX)][(4+SY)])?(RefPixels[7][4]-CompPixels [(7+SX)][(4+SY)]):(CompPixels [(7+SX)][(4+SY)]-RefPixels[7][4])
					+(RefPixels[7][5]>CompPixels [(7+SX)][(5+SY)])?(RefPixels[7][5]-CompPixels [(7+SX)][(5+SY)]):(CompPixels [(7+SX)][(5+SY)]-RefPixels[7][5])
					+(RefPixels[7][6]>CompPixels [(7+SX)][(6+SY)])?(RefPixels[7][6]-CompPixels [(7+SX)][(6+SY)]):(CompPixels [(7+SX)][(6+SY)]-RefPixels[7][6])
					+(RefPixels[7][7]>CompPixels [(7+SX)][(7+SY)])?(RefPixels[7][7]-CompPixels [(7+SX)][(7+SY)]):(CompPixels [(7+SX)][(7+SY)]-RefPixels[7][7])
					+ 1);		
					
					
					state<=calcsumabs2;
				end //end calcsums
				
				calcsumabs2:
				begin
					rLED[17:16]<=2'b10;
					if (SADTemp<SADMin)
					begin
						rLED[15]<=~rLED[15];
						SADMin <= SADTemp;
						SXMin<=SX;
						SYMin<=SY;
					end
					else if ((SADTemp == SADMin) && (SY==7) && (SX==7)) //no movement
					begin
						SADMin <= SADTemp;
						SXMin<=SX;
						SYMin<=SY;
					end
					if (doneproc) //the last sum is complete!
					begin
						state<=done;
						doneproc<=0;
					end
					else
						state<=calcsumabs;
				end
				
				done:
				begin
					rLED[17:16]<=2'b11;
					SADMin<=16'h0000;
					if (ImYcoord == 479) //done	
						state<=done; //wait to latch on
					else
						state<=grab;
				end
				
			endcase
					
			//------------------------------------------------------------------------
			//------------------------------------------------------------------------
			//------------------------------------------------------------------------
			//------------------------------------------------------------------------
			//------------------------------------------------------------------------
			//------------------------------------------------------------------------			
			//------------------------------------------------------------------------
								
			
			if (ImXcoord == 639)
			begin
				ImXcoord <= 0;
				if (ImYcoord == 479) //done				
				begin
					ImYcoord<=0; //reached end of FIFO buffer					
				end
				else
				begin
					ImYcoord <= ImYcoord + 1;
				end
			end
			else
				ImXcoord <= ImXcoord+1;
		end
	end



endmodule
